@@ -1,14 +1,15 @@
 package Lyrics::Fetcher::LyricWiki;
 
-# $Id: LyricWiki.pm 306 2008-03-18 18:37:14Z davidp $
+# $Id: LyricWiki.pm 724 2009-09-08 21:51:59Z davidp $
 
 use 5.005000;
 use strict;
 use warnings;
-use SOAP::Lite;
+use LWP::UserAgent;
+use HTML::TagParser;
 use Carp;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 # the HTTP User-Agent we'll send:
 our $AGENT = "Perl/Lyrics::Fetcher::LyricWiki $VERSION";
@@ -58,26 +59,39 @@ sub fetch {
             'fetch() called without artist and song');
         return;
     }
+
+    my $ua = LWP::UserAgent->new();
+    $ua->agent($AGENT);
+
+    my $url = join ':', map { s/\s+/_/; $_ } ($artist, $song);
+    my $resp = $ua->get("http://lyrics.wikia.com/lyrics/$url");
     
-    my $soap = new SOAP::Lite->service('http://lyricwiki.org/server.php?wsdl');
-    my $result = $soap->getSong($artist, $song);
-        
-    
-    unless ($result->{lyrics}) {
-        $Lyrics::Fetcher::Error = 'SOAP request failed';
+    if (!$resp->is_success) {
+        if ($resp->status_line =~ /404/) {
+            # Lyrics for this song not found
+            $Lyrics::Fetcher::Error = 'Lyrics not found';
+            return;
+        } else {
+            # Something else wrong, so return HTTP error description
+            $Lyrics::Fetcher::Error = "Failed to fetch - " . $resp->status_line;
+            return;
+        }
+    }
+
+    # OK, parse the HTML:
+    my $html = $resp->content;
+    $html =~ s{<br\s+/?>}{%newline%}gi;
+    my $parser = HTML::TagParser->new( $html );
+
+    if (my $lyricsdiv = $parser->getElementsByClassName('lyricbox')) {
+        $Lyrics::Fetcher::Error = 'OK';
+        my $lyrics = $lyricsdiv->innerText;
+        $lyrics =~ s/%newline%/\n/g;
+        return $lyrics;
+    } else {
+        $Lyrics::Fetcher::Error = 'No lyrics parsed from page';
         return;
     }
-    
-    if ($result->{lyrics} eq 'Not found') {
-        $Lyrics::Fetcher::Error = 
-                'Lyrics not found';
-            return;
-    }
-    
-    # looks like it worked:
-    $Lyrics::Fetcher::Error = 'OK';
-    return $result->{lyrics};
-
 
 }
 
@@ -96,9 +110,8 @@ If you find any bugs, please let me know.
 
 =head1 THANKS
 
-Thanks to Sean Colombo for creating www.LyricWiki.org, and for creating
-SOAP-based web services to fetch lyrics (that's *so* much nicer than having
-to screen-scrape them!).
+Thanks to Sean Colombo for creating www.LyricWiki.org, and thanks to Wikia.com
+for taking on the project and keeping it alive.
 
 
 =head1 COPYRIGHT
@@ -107,6 +120,14 @@ This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 
+=head1 DISCLAIMER
+
+Lyrics are copyright their original owners and/or record companies who purchased
+the rights to them.  Lyrics accessed programmatically via this module should be
+used for educational purposes only, and should not be distributed/re-published
+without authorisation.  You are responsible for your usage of this module.
+
+
 =head1 AUTHOR
 
 David Precious E<lt>davidp@preshweb.co.ukE<gt>
@@ -115,7 +136,7 @@ David Precious E<lt>davidp@preshweb.co.ukE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007-2008 by David Precious
+Copyright (C) 2007-2009 by David Precious
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.7 or,
